/*
 * Copyright [2008]. Sherwin zhu. sherwinzhu@126.com
 *
 * http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.DirectoryServices;

namespace TB.ADBlock
{
    /// <summary>
    /// ADĻࡣ
    /// </summary>
    public class BaseObject
    {
        #region PROPERTY Const

        public const string PROPERTY_OBJECTCLASS    = "objectClass";            // ,string[]eg:user is {"top","person","organizationalPerson","user"}
        public const string PROPERTY_OBJECTCATEGORY = "objectCategory";         // ,stringeg:user is "CN=Person,CN=Schema,CN=Configuration,DC=maodou,DC=com"

        public const string PROPERTY_NAME           = "name";                   // RDN,ֱͨӸֵ޸,string

        public const string PROPERTY_OBJECTGUID     = "objectGUID";             // GUID,byte[16]ֽתΪ16ƣNativeGuidAD󶨣תΪGuidṹ
        public const string PROPERTY_OBJECTSID      = "objectSid";              // SID,byte[28]
        public const string PROPERTY_DN             = "distinguishedName";      // DN,string

        public const string PROPERTY_WHENCREATED    = "whenCreated";            // ʱ,DateTime
        public const string PROPERTY_WHENCHANGED    = "whenChanged";            // ʱ,DateTime
        public const string PROPERTY_INSTANCETYPE   = "instanceType";           // integ:user is 4
        public const string PROPERTY_USNCREATED     = "uSNCreated";             // ?,IADsLargeInteger
        public const string PROPERTY_USNCHANGED     = "uSNChanged";             // ?,IADsLargeInteger
        public const string PROPERTY_SECURITY       = "nTSecurityDescriptor";   // ?,?

        #endregion


        #region fields

        /// <summary>
        /// Guid
        /// </summary>
        protected Guid guid;
        /// <summary>
        /// SID
        /// </summary>
        protected byte[] objectSid;
        //Distinguished Name
        private string dn;
        /// <summary>
        /// Object Class
        /// </summary>
        protected string[] objectClass;
        /// <summary>
        /// ObjectCategory DN
        /// </summary>
        protected string objectCategory;
        /// <summary>
        /// Name
        /// </summary>
        protected string name;
        /// <summary>
        /// SchemaClass NameӦö٣
        /// </summary>
        protected SchemaClass schema;
        // path
        private string path;

        private DateTime whenCreated;

        // guid:227451df-dbe1-4988-9ed6-1523cdfcece9
        // nativeguid:df517422e1db88499ed61523cdfcece9
        #endregion


        #region properity

        /// <summary>
        /// Guid
        /// </summary>
        public Guid Guid
        {
            get { return guid; }
            set { guid = value; }
        }

        /// <summary>
        /// NativeGuidLDAP󶨲ѯGUID
        /// </summary>
        public string NativeGuid
        {
            get { return Utils.ConvertGuidToNativeGuid(this.guid); }
        }

        /// <summary>
        /// ObjectSID
        /// </summary>
        public byte[] ObjectSid
        {
            get { return objectSid; }
            set { objectSid = value; }
        }

        /// <summary>
        /// ObjectSID16ַʽLDAP󶨲ѯObjectSid
        /// </summary>
        public string OctetObjectSid
        {
            get { return Utils.ConvertToOctetString(objectSid); }
        }

        /// <summary>
        /// Distinguished Nameת巴б/ַ
        /// </summary>
        /// <remarks>ֱá</remarks>
        public string Dn
        {
            get { return dn; }
            set 
            { 
                //dn = value; 
            }
        }

        /// <summary>
        /// ADsPathThe fully qualified path 
        /// </summary>
        public string Path
        {
            get { return this.path; }
        }

        /// <summary>
        /// DNɵADsPath
        /// </summary>
        public string DNPath
        {
            get { return ParaMgr.LDAP_IDENTITY + this.dn; }
        }

        /// <summary>
        /// Name
        /// </summary>
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        /// <summary>
        /// SchemaClassName
        /// </summary>
        public string SchemaClassName
        {
            get
            {
                return this.schema.ToString("F");
            }
        }

        /// <summary>
        /// ObjectCategory Name
        /// </summary>
        public string ObjectCategory
        {
            get
            {
                return Utils.GetRDNValue(this.objectCategory);
            }
        }

        /// <summary>
        /// ȡʱ
        /// </summary>
        public DateTime WhenCreated
        {
            get
            {
                return this.whenCreated;
            }
        }

        #endregion


        #region .ctors

        /// <summary>
        /// ĬϹ캯ֱӴ
        /// </summary>
        protected BaseObject()
        { }

        #endregion


        /// <summary>
        /// DirectoryEntry
        /// </summary>
        /// <param name="entry">DirectoryEntry</param>
        protected virtual void Parse(DirectoryEntry entry)
        {
            this.ParsePrivate(entry);
        }

        private void ParsePrivate(DirectoryEntry entry)
        {
            this.guid = entry.Guid;
            this.dn = Utils.EscapeDNBackslashedChar(Utils.GetProperty(entry, BaseObject.PROPERTY_DN));     // ת巴б'/'ַ
            this.path = entry.Path;
            this.name = Utils.GetProperty(entry, BaseObject.PROPERTY_NAME);

            List<string> ocList = new List<string>();
            foreach (object oc in (object[])(entry.Properties[BaseObject.PROPERTY_OBJECTCLASS].Value))
            {
                ocList.Add(oc.ToString());
            }
            this.objectClass = ocList.ToArray();
            this.objectCategory = Utils.GetProperty(entry, BaseObject.PROPERTY_OBJECTCATEGORY);

            this.schema = (SchemaClass)(Enum.Parse(typeof(SchemaClass), entry.SchemaClassName));

            // OU ûobjectSid
            if (schema == SchemaClass.user || schema == SchemaClass.group || schema == SchemaClass.builtinDomain ||
                schema == SchemaClass.computer || schema == SchemaClass.domainDNS)
                this.objectSid = (byte[])(entry.Properties[BaseObject.PROPERTY_OBJECTSID].Value);
            else
                this.objectSid = new byte[] { };
        }

        /// <summary>
        /// DirectoryEntry
        /// </summary>
        /// <param name="entry">DirectoryEntry</param>
        /// <param name="demandSchema">ҪĶSchemaClassԱ͡</param>
        protected void Parse(DirectoryEntry entry, SchemaClass demandSchema)
        {
            // this.Parse(entry);      // ERROR -- εthis

            this.ParsePrivate(entry);

            if (this.schema != demandSchema)
            {
                entry.Close();
                entry.Dispose();

                throw new SchemaClassException("Ͳ" + demandSchema.ToString("F"));
            }
        }

        /// <summary>
        /// SearchResult
        /// </summary>
        /// <param name="result">SearchResultӦҪԡ</param>
        /// <param name="demandSchema">ҪĶSchemaClassԱ͡</param>
        protected void Parse(SearchResult result, SchemaClass demandSchema)
        {
            string nativeGuid = "";
            foreach (byte g in (byte[])result.Properties["objectguid"][0])
            {
                nativeGuid += g.ToString("x2");
            }
            this.guid = Utils.ConvertNativeGuidToGuid(nativeGuid);
            this.dn = Utils.EscapeDNBackslashedChar(Utils.GetProperty(result, "distinguishedname"));     // ת巴б'/'ַ

            this.path = Utils.GetProperty(result, "adspath");
            this.name = Utils.GetProperty(result, "name");

            List<string> ocList = new List<string>();
            foreach (object oc in result.Properties["objectclass"])
            {
                ocList.Add(oc.ToString());
            }
            this.objectClass = ocList.ToArray();
            this.objectCategory = Utils.GetProperty(result, "objectcategory");

            this.schema = SchemaClass.none;
            foreach(string oc in this.objectClass)
            {
                // ʱֻ
                switch(oc)
                {
                    case "organizationalUnit":
                        this.schema = SchemaClass.organizationalUnit;
                        break;
                    case "group":
                        this.schema = SchemaClass.group;
                        break;
                    case "user":
                        this.schema = SchemaClass.user;
                        break;

                }
                if (this.schema != SchemaClass.none)
                    break;
            }

            // OU ûobjectSid
            if (schema == SchemaClass.user || schema == SchemaClass.group)
                this.objectSid = (byte[])(result.Properties["objectsid"][0]);
            else
                this.objectSid = new byte[] { };


            this.whenCreated = DateTime.Parse(Utils.GetProperty(result, "whenCreated"));

            if (this.schema != demandSchema)
            {
                throw new SchemaClassException("Ͳ" + demandSchema.ToString("F"));
            }
        }

        ///<summary>
        ///ƶDirectoryEntryָλá
        ///</summary>
        ///<param name="newLocationPath">ƶλõADsPath</param>
        ///<param name="userName">ûݱʶ--ûΪʱʹĬûݱʶ</param>
        ///<param name="password">ûݱʶ--롣</param>
        public virtual void Move(string newLocationPath, string userName, string password)
        {
            DirectoryEntry de = ADManager.GetByDN(this.dn, userName, password);     // DN -- ADManager.Move˵

            ADManager.Move(de, newLocationPath, userName, password);

            this.Parse(de);

            de.Close();
            de.Dispose();
        }

        /// <summary>
        /// ƶDirectoryEntryָλãʹĬûݱʶ
        /// </summary>
        /// <param name="newLocationPath">ƶλõADsPath</param>
        public virtual void Move(string newLocationPath)
        {
            this.Move(newLocationPath, null, null);
        }


        #region Identity

        /// <summary>
        /// ִжʱʹõĬûݱʶ
        /// </summary>
        /// <param name="userName">ûݱʶ--û</param>
        /// <param name="password">ûݱʶ--롣</param>
        public void SetIdentity(string userName, string password)
        {
            this.hasIdentity = true;
            this.iUserName = userName;
            this.iPassword = password;
        }

        /// <summary>
        /// ִжʱʹõĬûݱʶ
        /// ʱʹִн̵ݱʶ
        /// </summary>
        public void ClearIdentity()
        {
            this.hasIdentity = false;
            this.iUserName = null;
            this.iPassword = null;
        }
        protected bool hasIdentity = false;
        protected string iUserName = null;
        protected string iPassword = null;

        #endregion
  
    }
}
